%=======================================================================
function load_edis
%=======================================================================
% loads a series of edi files and saves its contents in a mat file.
% ws, april '04
%======================================================================= 

% settings
%
ptol  = 0.05; % no longer needed (see below);
ndper = 8;
% -> periods resp. period bins per decade.

% file w/ edi-filenames of project
sname = 'can_edis.dat';

coord = []; pers = [];

fidsta=fopen(sname,'r');
% start loop over sites
ista = 0;
while 1
    line=fgetl(fidsta);
    if ~ischar(line) break; end
    
    ista=ista+1;
    edifile = sscanf(line,'%s');
    % read edi file
    [head,info,definemeas,mtsect,dat] = read_edi([edifile,'.edi']);
    % azimuth = negative declination
    theta       = -dat.azim;
    % LINE BELOW HAS TO BE COMMENTED OUT FOR NORMAL USE!!!
    theta = 0
    coordinates = dat.coord(1:2)';
    ntp         = length(dat.freq);
    period     = 1./dat.freq;
        
    % rotate impedances to geographic north
    [z,dz] = rot_z_edi(dat.z,dat.dz,theta);
    
    % same for geomagnetic transfer function
    [z_hz,dz_hz] = rot_hz_edi(dat.z_hz,dat.dz_hz,theta);
    
    % transform to app res and phi
    % [rho,drho,phi,dphi] = z2rho_edi(z,dz,1./dat.freq);

    % phase tensor + ~SVD according to Caldwell GJI 03
    [ph_tens,ph_svd,dph_tens,dph_svd] = ph_tensor_edi(z,dz,period);
    
    % copy everything into cell array
    zz{ista}    = z;          ee{ista} = dz;       
    zz_hz{ista} = z_hz;    ee_hz{ista} = dz_hz;
    coord = cat(2,coord,coordinates);
    per{ista} = period;
    pers = [pers,period];    
end
nsta=ista

% what periods have been found?
period=sort(pers);
period=[period(1), period(find(diff(period)>0)+1)]

% which ones do we want of these? -> ptol
%rel_diff=diff(period)./period(1:length(period)-1);
%period=[period(1), period(find(rel_diff>ptol)+1)]

% above: that would have been one approach.
% below is a more direct one 
% - spanning equal sized (log10) bins (presumably much better!)
lminp=log10(period(1)); lmaxp=log10(period(length(period))+.05);
lper=linspace(lminp,lmaxp,round((lmaxp-lminp)*ndper)+1);
period=10.^lper'
nper=length(period)

ZZZ(1:4,1:nper,1:nsta) = NaN;       EEE(1:4,1:nper,1:nsta) = NaN;
ZZZ_HZ(1:2,1:nper,1:nsta) = NaN;    EEE_HZ(1:2,1:nper,1:nsta) = NaN;

% to match frequencies (analogue get_freq)
% adjust data indices according to period and write matrices
for ista=1:nsta
    for iper=1:nper
        for itper=1:length(per{ista})
            logtper = log10(per{ista}(itper));
            logper = log10(period(iper));
            if abs(logtper-logper) < (1/ndper)/2
                ZZZ(:,iper,ista) = zz{ista}(:,itper);
                EEE(:,iper,ista) = ee{ista}(:,itper);
                ZZZ_HZ(:,iper,ista) = zz_hz{ista}(:,itper);
                EEE_HZ(:,iper,ista) = ee_hz{ista}(:,itper);
                break;
            end
        end
    end
end

% store in MAT file
T = period';
coord = [coord(2,:);coord(1,:)]';
save HZSE_edi_abc ZZZ EEE ZZZ_HZ EEE_HZ T coord
%save HZSE_edi_abc coord


%=======================================================================
function [z,dz]=rot_z_edi(z,dz,theta)
%=======================================================================
% rotates impedances w/ errors as found in edi file
% theta is in degrees (positive is clockwise)

c = cos(theta*pi/180.);
s = sin(theta*pi/180.);

U = [ c*c ,  c*s ,  c*s , s*s  ;...
     -c*s ,  c*c , -s*s , c*s  ;...
     -c*s , -s*s ,  c*c , c*s  ;...
      s*s , -c*s , -c*s , c*c ];

z = U*z;
dz = (U.^2*dz.^2).^.5;

return

%================================================================
function [z_hz,dz_hz] = rot_hz_edi(z_hz,dz_hz,theta)
%================================================================
% rotates local magnetic transfer functions (induction vectors)
% as found in edi file 
% theta in degrees (positive is clockwise) 

c = cos(theta*pi/180.);
s = sin(theta*pi/180.);

Uv = [ c  , s ;...
      -s  , c ];

z_hz = Uv*z_hz;
dz_hz = (Uv.^2*dz_hz.^2).^.5;

return
