%==========================================================================
%
function [head,info,definemeas,mtsect,dat] = read_edi(file)
%
%==========================================================================
% this is a first start to get edi files read into matlab
% - just a few realizations of the 'standard' will be readable!
%
% section: '>='
% key:     '>'
% param:   '='
%
% get_key/section   : store lines of key/section in variable
% read_section      : read these lines (section) into two structs
% read_mtsect (e.g.): read these structs
% 
% ws, april '04
%==========================================================================

if exist(file,'file')
    fid=fopen(file,'r');
    disp(['File ',file,' opened']);
else
    error(['File ',file,' not found!']);
    return
end

%============================================
head = get_key(fid,0,'HEAD');
%============================================
info = get_key(fid,0,'INFO');
%============================================
definemeas = get_section(fid,'DEFINEMEAS');
[param,key] = read_section(fid,definemeas);
[coord] = read_definemeas(param,key);
%============================================
mtsect = get_section(fid,'MTSECT');
[param,key] = read_section(fid,mtsect);
[freq,azim,z,dz,z_hz,dz_hz] = read_mtsect(param,key);
%============================================

dat = struct(...
    'freq',freq,'azim',azim,'coord',coord,'z',z,'dz',dz,...
    'z_hz',z_hz,'dz_hz',dz_hz);

return


%============================================
function coord = read_definemeas(param,key)
%============================================
par = {'UNITS','REFLAT','REFLONG','REFELEV'};

for ipar=1:length(param)
    for kpar=1:length(par)
        if strmatch(param(ipar).name,char(par(kpar)))
            parval{kpar}=param(ipar).value;
        end
    end
end

units=char(parval{1});
if units(1:1)~='M'
    warning(['units are non-metric: units=',units]); 
end

lat=parval{2}; lon=parval{3}; ele=str2num(parval{4});
if ~isempty(lat) & ~isempty(lon)
    [dms]=sscanf(lat,'%d:%d:%d');
    rest=(dms(2)+dms(3)/60)/60;
    if dms(1)<0 lat=dms(1)-rest; else lat=dms(1)+rest; end  
    [dms]=sscanf(lon,'%d:%d:%d');
    rest=(dms(2)+dms(3)/60)/60;
    if dms(1)<0 lon=dms(1)-rest; else lon=dms(1)+rest; end  
    coord=[lat,lon,ele];
else
    coord=[0 0 0];
end
return

%============================================
function [freq,azim,z,dz,z_hz,dz_hz] = read_mtsect(param,key)
%============================================
par={'NFREQ','SECTID'};

for ipar=1:length(param)
    for kpar=1:length(par)
        if strmatch(param(ipar).name,char(par(kpar)))
            parval{kpar}=param(ipar).value;
        end
    end
end
nfreq=str2num(parval{1});
sectid=parval{2}(find(parval{2}~='"'));

for ikey=1:length(key)
    if findstr(key(ikey).name,'FREQ')
        disp('found entry FREQ');
        freq=str2num(key(ikey).value);
    end 
end 
for ikey=1:length(key)
    if findstr(key(ikey).name,'ZROT')
        disp('found entry ZROT')
        zrot=str2num(key(ikey).value);
        if diff(zrot)~= 0 
            warning('impedances have varying azimuth!'); 
        end
        azim_z=zrot(1);  
    end 
end

for ikey=1:length(key)
    if findstr(key(ikey).name,'RHOROT')
        disp('found entry RHOROT')
        rhorot=str2num(key(ikey).value);
        if diff(rhorot)~= 0 
            warning('apparent resistivities have varying azimuth!'); 
        end
        azim_rho=rhorot(1);  
    end 
end

imp = ['ZXX';'ZXY';'ZYX';'ZYY'];
riv = {'R';'I';'.VAR'};
limp=0; 
for iimp=1:4
    for iriv=1:3
        for ikey=1:length(key)
            entry = [imp(iimp,:),char(riv(iriv,:))];
	        if strmatch(key(ikey).name,entry)
                disp(['found entry ',entry]);
	            mimp(iimp,iriv,:)=str2num(key(ikey).value);
                if key(ikey).param.name=='ROT' 
                    rot=key(ikey).param.value;
                    if strmatch('NORTH',rot)
                        azim=0;
                    elseif strmatch('ZROT',rot)
                        azim=azim_z;
                    elseif strmatch('RHOROT',rot)
                        azim=azim_rho;
                    end
                else
                    azim=0;
                end
            end 
        end          
    end 
end    
if exist('mimp','var')
    limp=1; % no longer used
    z = squeeze(mimp(:,1,:) + i*mimp(:,2,:));
    %z = reshape(z,2,2,[]);
    % var=variance, convert to error=standard deviation
    dz = squeeze((mimp(:,3,:)).^.5);
    %dz = reshape(dz,2,2,[]);
end   

cij    = ['XX';'XY';'YX';'YY'];
crhoph = ['RHO';'PHS'];
suff   = {'';'.ERR'};
lrhophs=0;
for ij=1:4
    for irp=1:2
        for isuff=1:2
            entry = [crhoph(irp,:),cij(ij,:),char(suff(isuff,:))];
            for ikey=1:length(key)
                if strmatch(key(ikey).name,entry,'exact')
                    disp(['found entry ',entry]);
                    mrhophs(ij,irp,isuff,:)=str2num(key(ikey).value);
                    if key(ikey).param.name=='ROT' 
                        rot=key(ikey).param.value;
                        if strmatch('NORTH',rot)
                            azim=0;
                        elseif strmatch('ZROT',rot)
                            azim=azim_z;
                        elseif strmatch('RHOROT',rot)
                            azim=azim_rho;
                        end
                    else
                        azim=0;
                    end
                end 
            end          
        end 
    end
end
if exist('mrhophs','var') & ~exist('mimp','var')
    lrhophs=1; % no longer used
    srhophs=size(mrhophs);
    % YY entry not found? [XX will always be 'there']
    if srhophs(1)==3
        mrhophs(4,:,:,:)=zeros([1 srhophs(2:4)]);
    end
    % convert to impedances right away - we want to rotate!
    % rho = mu0*omega*|C|^2 = mu0/omega*|Z|^2 = 2*10^-7/f*|Z|^2
    % w/ Z in km/s: rho=2*10-1*T*|Z|^2
    % => z = 5*f*sqrt(rho)
    z = squeeze(...
          sqrt(mrhophs(:,1,1,:)).*cos(mrhophs(:,2,1,:)*pi/180) + ... % real
        i*sqrt(mrhophs(:,1,1,:)).*sin(mrhophs(:,2,1,:)*pi/180)) ... % imaginary
        .*sqrt(5*(ones(4,1)*freq));
    %z = reshape(z,2,2,[]);
    % error found is in standard deviation
    % requested: circle in complex plane around each impedance
    % dz = sqrt(drho^2+(rho*dphs)^2)*5*f
    dz = sqrt(squeeze(...
            mrhophs(:,1,2,:).^2 + ...
           (mrhophs(:,1,1,:) * pi/180 .* mrhophs(:,2,2,:)).^2)) ...
         .*sqrt(5*(ones(4,1)*freq));
    %dz = reshape(dz,2,2,[]);
end   

tip = ['TX';'TY'];
riv = {'R.EXP';'I.EXP';'VAR.EXP'};
ltip=0;
for itip=1:2
    for iriv=1:3
        entry = [tip(itip,:),char(riv(iriv,:))];
        for ikey=1:length(key)
	        if strmatch(key(ikey).name,entry)
                disp(['found entry ',entry]);
	            mtip(itip,iriv,:)=str2num(key(ikey).value);
                if key(ikey).param.name=='ROT'
                    rot=key(ikey).param.value;
                    if rot=='NORTH' azim=0; end
                end 
            end  
        end           
    end 
end 
if exist('mtip','var')
    ltip=1;
    z_hz = squeeze(mtip(:,1,:) + i*mtip(:,2,:));
    dz_hz = squeeze((mtip(:,3,:)).^5);
else
    z_hz = zeros(2,nfreq);
    dz_hz = zeros(2,nfreq);
end    
% note: different azimuths in T and Z keys are not considered.
% RHO entries are not implemented yet.

return

%============================================
function [param,key] = read_section(fid,section)
%============================================
% reads the lines of a section (output from get_section)
% creates structure arrays 'param' and 'key'
%
% |param(=)         |key(>)    
%  |pname |pvalue    |kname |kvalue |kparam(=)
%                                    |kpname |kpvalue

iparam=0; ikey=0; iskey=0; vline=[];

param=struct('name',[],'value',[]);
key  =struct('name',[],'value',[],...
             'param',struct('name',[],'value',[]));

for iline=2:length(section)
    line=char(section{iline});
    if line(1:1)~='>'
        % read parameters or continue reading key
        if iskey==1
            vline=strcat(vline,'NEWLINE',line);
        elseif sum(findstr(line,'='))>0
            iparam=iparam+1;
            lindex=findstr(line,'='); line(lindex)=' ';
            rest=line;
            for ipar=iparam:length(lindex)+iparam-1
                [name,rest]=strtok(rest);
                param(ipar).name=name; 
                [value,rest]=strtok(rest);   
                param(ipar).value=value;
            end  
        end  
    elseif line(1:2)=='>!'
        % comment line => nothing to do!
    else
        % so it's '>' plus a keyword
        % store value of key before in case there is any (//)
        if iskey==1
            % replace newlines
            mnl=strfind(vline,'NEWLINE');
            for inl=1:length(mnl)
                vline(mnl(inl):mnl(inl)+6)='       ';
            end
            key(ikey).value=vline; 
            vline=[]; iskey=0;
        end
        if findstr(line,'>END') return; end
        ikey=ikey+1;
        if findstr(line,'//') iskey=1; end
        line(1:1)=' ';
        lindex=findstr(line,'='); line(lindex)=' ';
        [name,rest]=strtok(line);
        key(ikey).name=name;
        for ikpar=1:length(lindex)
            [name,rest]=strtok(rest);
            key(ikey).param.name=name;
            [value,rest]=strtok(rest);
            key(ikey).param.value=value;
        end          
    end    
end  
 
return

%============================================
function value = get_section(fid,section)
%============================================
% gets the line(s) of a section (>=)
fseek(fid,0,-1);
kline=1; line=1;
while line~=-1
    line=fgets(fid);
    if findstr(line,['>=',section])
        value{1}=line;
        line = fgets(fid);
        while line~=-1 & sum(findstr(line,'>='))==0
            kline=kline+1;
            value{kline}=line;
            line = fgets(fid);
        end  
        return 
    end 
end 
warning(['section ',section,' not found']);
return

%============================================
function value = get_key(fid,pos,key)
%============================================
% reads in the line(s) belonging to a '>' key
% starts searching at pos.
fseek(fid,pos,-1);
kline=1; line=1;
while line~=-1
    line=fgets(fid);
    if findstr(line,['>',key])
        value{1}=line;
        line = fgets(fid);
        while line~=-1 & line(1:1)~='>'
            kline=kline+1;
            value{kline}=line;
            line = fgets(fid);
        end  
        return
    end  
end  
warning(['keyword ',key,' not found']);
return
